﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Text;

namespace Understanding_Linq_3a
{
    public class Projector<TSource, TTarget>  : IEnumerable<TTarget> where TSource : new()
    {
        public Projector(Filter<TSource> source, Expression<Func<TSource, TTarget>> projection)
        {
            m_source = source;
            m_projection = projection;
        }

        Filter<TSource> m_source;
        Expression<Func<TSource, TTarget>> m_projection;

        private string ExtractColumnsFromProjection()
        {
            //use helper to analyze expression
            Expressions.CollectColumnAccess helper = new Understanding_Linq_3a.Expressions.CollectColumnAccess(m_projection);

            return helper.ColumnsAccessed.ToArray().ToCsvString(",");
        }

        #region IEnumerable<TTarget> Members

        public IEnumerator<TTarget> GetEnumerator()
        {
            //initialize remote executing
            RemoteExecutor executor = new RemoteExecutor();
            executor.ColumnsToLoad = ExtractColumnsFromProjection();

            //compile expression to delegate
            Func<TSource, TTarget> projectionDelegate = m_projection.Compile();

            foreach (TSource current in m_source.GetRemote(executor))
            {
                yield return projectionDelegate(current);
            }
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion
    }
}
